@@ -1,5 +1,5 @@
-;;;; $Id: allegro.lisp 687 2012-02-27 14:49:55Z ctian $
-;;;; $URL: svn://common-lisp.net/project/usocket/svn/usocket/tags/0.5.5/backend/allegro.lisp $
+;;;; $Id$
+;;;; $URL$
 
 ;;;; See LICENSE for licensing information.
 
@@ -151,10 +151,16 @@
   (values (get-peer-address usocket)
           (get-peer-port usocket)))
 
-(defmethod socket-send ((socket datagram-usocket) buffer length &key host port)
-  (with-mapped-conditions (socket)
-    (let ((s (socket socket)))
-      (socket:send-to s buffer length :remote-host host :remote-port port))))
+(defmethod socket-send ((usocket datagram-usocket) buffer size &key host port (offset 0))
+  (with-mapped-conditions (usocket)
+    (let ((s (socket usocket)))
+      (socket:send-to s
+		      (if (zerop offset)
+			  buffer
+			  (subseq buffer offset (+ offset size)))
+		      size
+		      :remote-host host
+		      :remote-port port))))
 
 (defmethod socket-receive ((socket datagram-usocket) buffer length &key)
   (declare (values (simple-array (unsigned-byte 8) (*)) ; buffer
