@@ -1,10 +1,14 @@
-;;;; $Id: openmcl.lisp 687 2012-02-27 14:49:55Z ctian $
-;;;; $URL: svn://common-lisp.net/project/usocket/svn/usocket/tags/0.5.5/backend/openmcl.lisp $
+;;;; $Id$
+;;;; $URL$
 
 ;;;; See LICENSE for licensing information.
 
 (in-package :usocket)
 
+(eval-when (:compile-toplevel :load-toplevel :execute)
+  (when (find-class 'ccl::ip6-socket-address nil)
+    (pushnew :ccl-1.11-sockets *features*)))
+
 (defun get-host-name ()
   (ccl::%stack-block ((resultbuf 256))
     (when (zerop (#_gethostname resultbuf 256))
@@ -42,16 +46,17 @@
       (let ((max-fd -1))
         (dolist (sock sockets)
           (let ((fd (openmcl-socket:socket-os-fd (socket sock))))
-            (setf max-fd (max max-fd fd))
-            (ccl::fd-set fd infds)))
-        (let* ((res (#_select (1+ max-fd)
-                              infds (ccl::%null-ptr) (ccl::%null-ptr)
-                              (if ticks-to-wait tv (ccl::%null-ptr)))))
+            (when fd ;; may be NIL if closed
+              (setf max-fd (max max-fd fd))
+              (ccl::fd-set fd infds))))
+        (let ((res (#_select (1+ max-fd)
+                             infds (ccl::%null-ptr) (ccl::%null-ptr)
+                             (if ticks-to-wait tv (ccl::%null-ptr)))))
           (when (> res 0)
-            (dolist (x sockets)
-              (when (ccl::fd-is-set (openmcl-socket:socket-os-fd (socket x))
-                                    infds)
-                (setf (state x) :READ))))
+            (dolist (sock sockets)
+              (let ((fd (openmcl-socket:socket-os-fd (socket sock))))
+                (when (and fd (ccl::fd-is-set fd infds))
+                  (setf (state sock) :READ)))))
           sockets)))))
 
 (defun raise-error-from-id (condition-id socket real-condition)
@@ -74,15 +79,50 @@
 	      (nameserver-error (cdr (assoc condition-id
 					    +openmcl-nameserver-error-map+))))
 	 (if nameserver-error
-	   (error nameserver-error :host-or-ip nil)
+             (if (typep nameserver-error 'serious-condition)
+                 (error nameserver-error :host-or-ip nil)
+                 (signal nameserver-error :host-or-ip nil))
 	   (raise-error-from-id condition-id socket condition))))))
 
-(defun to-format (element-type)
-  (if (subtypep element-type 'character)
-      :text
-    :binary))
+(defun to-format (element-type protocol)
+  (cond ((null element-type)
+	 (ecase protocol ; default value of different protocol
+	   (:stream :text)
+	   (:datagram :binary)))
+	((subtypep element-type 'character)
+	 :text)
+	(t :binary)))
 
-(defun socket-connect (host port &key (protocol :stream) (element-type 'character)
+#+ccl-1.11-sockets
+(defun socket-connect (host port &key (protocol :stream) element-type
+                                   timeout deadline nodelay
+                                   local-host local-port)
+  (when (eq nodelay :if-supported)
+    (setf nodelay t))
+  (with-mapped-conditions ()
+    (let* ((remote (when (and host port)
+                     (openmcl-socket:resolve-address :host (host-to-hostname host)
+						     :port port
+						     :socket-type protocol)))
+           (mcl-sock (openmcl-socket:make-socket :type protocol
+                                                 :address-family (openmcl-socket:socket-address-family remote)
+                                                 :remote-address remote
+                                                 :local-host (host-to-hbo local-host)
+                                                 :local-port local-port
+                                                 :format (to-format element-type protocol)
+                                                 :external-format ccl:*default-external-format*
+                                                 :deadline deadline
+                                                 :nodelay nodelay
+                                                 :connect-timeout timeout
+                                                 :input-timeout timeout)))
+      (ecase protocol
+        (:stream
+         (make-stream-socket :stream mcl-sock :socket mcl-sock))
+        (:datagram
+         (make-datagram-socket mcl-sock :connected-p (and remote t)))))))
+
+#-ccl-1.11-sockets
+(defun socket-connect (host port &key (protocol :stream) element-type
 		       timeout deadline nodelay
                        local-host local-port)
   (when (eq nodelay :if-supported)
@@ -91,11 +131,12 @@
     (ecase protocol
       (:stream
        (let ((mcl-sock
-	      (openmcl-socket:make-socket :remote-host (host-to-hostname host)
+	      (openmcl-socket:make-socket :remote-host host
 					  :remote-port port
-					  :local-host (when local-host (host-to-hostname local-host))
+					  :local-host local-host
 					  :local-port local-port
-					  :format (to-format element-type)
+					  :format (to-format element-type protocol)
+					  :external-format ccl:*default-external-format*
 					  :deadline deadline
 					  :nodelay nodelay
 					  :connect-timeout timeout)))
@@ -104,10 +145,11 @@
        (let* ((mcl-sock
                (openmcl-socket:make-socket :address-family :internet
                                            :type :datagram
-                                           :local-host (when local-host (host-to-hostname local-host))
+                                           :local-host local-port
                                            :local-port local-port
 					   :input-timeout timeout
-                                           :format :binary))
+					   :format (to-format element-type protocol)
+					   :external-format ccl:*default-external-format*))
               (usocket (make-datagram-socket mcl-sock)))
 	 (when (and host port)
 	   (ccl::inet-connect (ccl::socket-device mcl-sock)
@@ -116,8 +158,27 @@
 	 (setf (connected-p usocket) t)
 	 usocket)))))
 
+#+ccl-1.11-sockets
 (defun socket-listen (host port
-                           &key reuseaddress
+                      &key
+                        (reuse-address nil reuse-address-supplied-p)
+                        (reuseaddress (when reuse-address-supplied-p reuse-address))
+                        (backlog 5)
+                        (element-type 'character))
+  (let ((local-address (openmcl-socket:resolve-address :host (host-to-hostname host)
+						       :port port :connect :passive)))
+    (with-mapped-conditions ()
+      (make-stream-server-socket (openmcl-socket:make-socket :connect :passive
+                                                             :address-family (openmcl-socket:socket-address-family local-address)
+                                                             :local-address local-address
+                                                             :reuse-address reuseaddress
+                                                             :backlog backlog
+                                                             :format (to-format element-type :stream))
+                                 :element-type element-type))))
+
+#-ccl-1.11-sockets
+(defun socket-listen (host port
+                      &key reuseaddress
                            (reuse-address nil reuse-address-supplied-p)
                            (backlog 5)
                            (element-type 'character))
@@ -129,7 +190,7 @@
                                        :reuse-address reuseaddress
                                        :local-port port
                                        :backlog backlog
-                                       :format (to-format element-type))
+                                       :format (to-format element-type :stream))
                                  (when (ip/= host *wildcard-host*)
                                    (list :local-host real-host)))))))
     (make-stream-server-socket sock :element-type element-type)))
@@ -149,28 +210,46 @@
   (with-mapped-conditions (usocket)
     (close (socket usocket))))
 
-(defmethod socket-send ((usocket datagram-usocket) buffer length &key host port)
+(defmethod socket-send ((usocket datagram-usocket) buffer size &key host port (offset 0))
   (with-mapped-conditions (usocket)
     (if (and host port)
-	(openmcl-socket:send-to (socket usocket) buffer length
+	(openmcl-socket:send-to (socket usocket) buffer size
 				:remote-host (host-to-hbo host)
-				:remote-port port)
-	;; following functino was defined in "vendor/ccl-send.lisp"
-	(ccl::send-for-usocket (socket usocket) buffer length))))
+				:remote-port port
+				:offset offset)
+	;; Clozure CL's socket function SEND-TO doesn't support operations on connected UDP sockets,
+	;; so we have to define our own.
+	(let* ((socket (socket usocket))
+	       (fd (ccl::socket-device socket)))
+	  (multiple-value-setq (buffer offset)
+	    (ccl::verify-socket-buffer buffer offset size))
+	  (ccl::%stack-block ((bufptr size))
+	    (ccl::%copy-ivector-to-ptr buffer offset bufptr 0 size)
+	    (ccl::socket-call socket "send"
+	      (ccl::with-eagain fd :output
+		(ccl::ignoring-eintr
+		  (ccl::check-socket-error (#_send fd bufptr size 0))))))))))
 
 (defmethod socket-receive ((usocket datagram-usocket) buffer length &key)
   (with-mapped-conditions (usocket)
     (openmcl-socket:receive-from (socket usocket) length :buffer buffer)))
 
+(defun usocket-host-address (address)
+  (cond
+    ((integerp address)
+     (hbo-to-vector-quad address))
+    ((and (arrayp address)
+          (= (length address) 16)
+          (every #'= address #(0 0 0 0 0 0 0 0 0 0 #xff #xff)))
+     (make-array 4 :displaced-to address :displaced-index-offset 12))
+    (t
+     address)))
+
 (defmethod get-local-address ((usocket usocket))
-  (let ((address (openmcl-socket:local-host (socket usocket))))
-    (when address
-      (hbo-to-vector-quad address))))
+  (usocket-host-address (openmcl-socket:local-host (socket usocket))))
 
 (defmethod get-peer-address ((usocket stream-usocket))
-  (let ((address (openmcl-socket:remote-host (socket usocket))))
-    (when address
-      (hbo-to-vector-quad address))))
+  (usocket-host-address (openmcl-socket:remote-host (socket usocket))))
 
 (defmethod get-local-port ((usocket usocket))
   (openmcl-socket:local-port (socket usocket)))
@@ -211,3 +290,29 @@
       (input-available-p (wait-list-waiters wait-list)
 			 (when timeout ticks-timeout))
       wait-list)))
+
+;;; Helper functions for option.lisp
+
+(defun get-socket-option-reuseaddr (socket)
+  (ccl::int-getsockopt (ccl::socket-device socket)
+                                  #$SOL_SOCKET #$SO_REUSEADDR))
+
+(defun set-socket-option-reuseaddr (socket value)
+  (ccl::int-setsockopt (ccl::socket-device socket)
+                                  #$SOL_SOCKET #$SO_REUSEADDR value))
+
+(defun get-socket-option-broadcast (socket)
+  (ccl::int-getsockopt (ccl::socket-device socket)
+                                  #$SOL_SOCKET #$SO_BROADCAST))
+
+(defun set-socket-option-broadcast (socket value)
+  (ccl::int-setsockopt (ccl::socket-device socket)
+                                  #$SOL_SOCKET #$SO_BROADCAST value))
+
+(defun get-socket-option-tcp-nodelay (socket)
+  (ccl::int-getsockopt (ccl::socket-device socket)
+                                  #$IPPROTO_TCP #$TCP_NODELAY))
+
+(defun set-socket-option-tcp-nodelay (socket value)
+  (ccl::int-setsockopt (ccl::socket-device socket)
+                                  #$IPPROTO_TCP #$TCP_NODELAY value))
