@@ -1,18 +1,11 @@
-;;;; $Id: sbcl.lisp 687 2012-02-27 14:49:55Z ctian $
-;;;; $URL: svn://common-lisp.net/project/usocket/svn/usocket/tags/0.5.5/backend/sbcl.lisp $
+;;;; -*- Mode: Lisp -*-
+;;;; $Id$
+;;;; $URL$
 
 ;;;; See LICENSE for licensing information.
 
 (in-package :usocket)
 
-;; There's no way to preload the sockets library other than by requiring it
-;;
-;; ECL sockets has been forked off sb-bsd-sockets and implements the
-;; same interface. We use the same file for now.
-#+ecl
-(eval-when (:compile-toplevel :load-toplevel :execute)
-  (require :sockets))
-
 #+sbcl
 (progn
   #-win32
@@ -34,7 +27,7 @@
          (when (= result 0)
            (sb-alien:cast buf sb-alien:c-string))))))
 
-#+ecl
+#+(and ecl (not ecl-bytecmp))
 (progn
   #-:wsock
   (ffi:clines
@@ -51,7 +44,7 @@
    #+:msvc "#include <time.h>"
    #-:msvc "#include <sys/time.h>"
    "#include <ecl/ecl-inl.h>")
-
+#|
   #+:prefixed-api
   (ffi:clines
    "#define CONS(x, y) ecl_cons((x), (y))"
@@ -60,7 +53,7 @@
   (ffi:clines
    "#define CONS(x, y) make_cons((x), (y))"
    "#define MAKE_INTEGER(x) make_integer((x))")
-
+|#
   (defun fd-setsize ()
     (ffi:c-inline () () :fixnum
      "FD_SETSIZE" :one-liner t))
@@ -398,11 +391,14 @@ happen. Use with care."
   (with-mapped-conditions (usocket)
     (close (socket-stream usocket))))
 
-(defmethod socket-send ((socket datagram-usocket) buffer length &key host port)
-  (with-mapped-conditions (socket)
-    (let* ((s (socket socket))
-           (dest (if (and host port) (list (host-to-vector-quad host) port) nil)))
-      (sb-bsd-sockets:socket-send s buffer length :address dest))))
+(defmethod socket-send ((usocket datagram-usocket) buffer size &key host port (offset 0))
+  (with-mapped-conditions (usocket)
+    (let* ((s (socket usocket))
+	   (dest (if (and host port) (list (host-to-vector-quad host) port) nil))
+	   (real-buffer (if (zerop offset)
+			    buffer
+			    (subseq buffer offset (+ offset size)))))
+      (sb-bsd-sockets:socket-send s real-buffer size :address dest))))
 
 (defmethod socket-receive ((socket datagram-usocket) buffer length
 			   &key (element-type '(unsigned-byte 8)))
@@ -489,11 +485,13 @@ happen. Use with care."
 ;;; Based on LispWorks version written by Erik Huelsmann.
 
 #+win32 ; shared by ECL and SBCL
-(progn
+(eval-when (:compile-toplevel :load-toplevel :execute)
   (defconstant +wsa-wait-failed+ #xffffffff)
   (defconstant +wsa-wait-event-0+ 0)
-  (defconstant +wsa-wait-timeout+ 258)
+  (defconstant +wsa-wait-timeout+ 258))
 
+#+win32 ; shared by ECL and SBCL
+(progn
   (defconstant fd-read 1)
   (defconstant fd-read-bit 0)
   (defconstant fd-write 2)
@@ -529,6 +527,36 @@ happen. Use with care."
 
   (defun waiting-required (sockets)
     (notany #'socket-ready-p sockets))
+
+  (defun raise-usock-err (errno &optional socket)
+    (error 'unknown-error
+           :socket socket
+           :real-error errno))
+
+  (defun wait-for-input-internal (wait-list &key timeout)
+    (when (waiting-required (wait-list-waiters wait-list))
+      (let ((rv (wsa-wait-for-multiple-events 1 (wait-list-%wait wait-list)
+                                              nil (truncate (* 1000 timeout)) nil)))
+        (ecase rv
+          ((#.+wsa-wait-event-0+)
+           (update-ready-and-state-slots (wait-list-waiters wait-list)))
+          ((#.+wsa-wait-timeout+)) ; do nothing here
+          ((#.+wsa-wait-failed+)
+           (maybe-wsa-error rv))))))
+
+  (defun %add-waiter (wait-list waiter)
+    (let ((events (etypecase waiter
+                    (stream-server-usocket (logior fd-connect fd-accept fd-close))
+                    (stream-usocket (logior fd-read))
+                    (datagram-usocket (logior fd-read)))))
+      (maybe-wsa-error
+       (wsa-event-select (os-socket-handle waiter) (os-wait-list-%wait wait-list) events)
+       waiter)))
+
+  (defun %remove-waiter (wait-list waiter)
+    (maybe-wsa-error
+     (wsa-event-select (os-socket-handle waiter) (os-wait-list-%wait wait-list) 0)
+     waiter))
 ) ; progn
 
 #+(and sbcl win32)
@@ -553,10 +581,6 @@ happen. Use with care."
   (sb-alien:define-alien-routine ("WSACreateEvent" wsa-event-create)
       ws-event) ; return type only
 
-  (sb-alien:define-alien-routine ("WSAResetEvent" wsa-event-reset)
-      (boolean #.sb-vm::n-machine-word-bits)
-    (event-object ws-event))
-
   (sb-alien:define-alien-routine ("WSACloseEvent" wsa-event-close)
       (boolean #.sb-vm::n-machine-word-bits)
     (event-object ws-event))
@@ -587,11 +611,6 @@ happen. Use with care."
     (cmd sb-alien:long)
     (argp (* sb-alien:unsigned-long)))
 
-  (defun raise-usock-err (errno socket)
-    (error 'unknown-error
-           :socket socket
-           :real-error errno))
-
   (defun maybe-wsa-error (rv &optional socket)
     (unless (zerop rv)
       (raise-usock-err (sockint::wsa-get-last-error) socket)))
@@ -607,19 +626,6 @@ happen. Use with care."
         (when (plusp int-ptr)
           (setf (state socket) :read)))))
 
-  (defun wait-for-input-internal (wait-list &key timeout)
-    (when (waiting-required (wait-list-waiters wait-list))
-      (let ((rv (wsa-wait-for-multiple-events 1 (wait-list-%wait wait-list)
-                                              nil (truncate (* 1000 timeout)) nil)))
-        (ecase rv
-          ((#.+wsa-wait-event-0+)
-           (update-ready-and-state-slots (wait-list-waiters wait-list)))
-          ((#.+wsa-wait-timeout+)) ; do nothing here
-          ((#.+wsa-wait-failed+)
-           (raise-usock-err
-            (sb-win32::get-last-error-message (sb-win32::get-last-error))
-            wait-list))))))
-
   (defun map-network-events (func network-events)
     (let ((event-map (sb-alien:slot network-events 'network-events))
           (error-array (sb-alien:slot network-events 'error-code)))
@@ -682,19 +688,6 @@ happen. Use with care."
 			   (unless (null alien)
 			     (sb-alien:free-alien alien))))))
 
-  (defun %add-waiter (wait-list waiter)
-    (let ((events (etypecase waiter
-                    (stream-server-usocket (logior fd-connect fd-accept fd-close))
-                    (stream-usocket (logior fd-read))
-                    (datagram-usocket (logior fd-read)))))
-      (maybe-wsa-error
-       (wsa-event-select (os-socket-handle waiter) (os-wait-list-%wait wait-list) events)
-       waiter)))
-
-  (defun %remove-waiter (wait-list waiter)
-    (maybe-wsa-error
-     (wsa-event-select (os-socket-handle waiter) (os-wait-list-%wait wait-list) 0)
-     waiter))
 ) ; progn
 
 #+(and ecl (not win32))
@@ -721,7 +714,7 @@ happen. Use with care."
     (declare (ignore wl w)))
 ) ; progn
 
-#+(and ecl win32)
+#+(and ecl win32 (not ecl-bytecmp))
 (progn
   (defun maybe-wsa-error (rv &optional syscall)
     (unless (zerop rv)
