@@ -1,5 +1,5 @@
-;;;; $Id: condition.lisp 564 2010-09-28 09:15:13Z ctian $
-;;;; $URL: svn://common-lisp.net/project/usocket/svn/usocket/tags/0.5.5/condition.lisp $
+;;;; $Id$
+;;;; $URL$
 
 ;;;; See LICENSE for licensing information.
 
@@ -171,8 +171,8 @@ error available."))
     ,@body))
 
 (defparameter +unix-errno-condition-map+
-  `(((11) . retry-condition) ;; EAGAIN
-    ((35) . retry-condition) ;; EDEADLCK
+  `(((11) . ns-try-again-condition) ;; EAGAIN
+    ((35) . ns-try-again-condition) ;; EDEADLCK
     ((4) . interrupted-condition))) ;; EINTR
 
 (defparameter +unix-errno-error-map+
