@@ -1,5 +1,5 @@
-;;;; $Id: usocket.lisp 687 2012-02-27 14:49:55Z ctian $
-;;;; $URL: svn://common-lisp.net/project/usocket/svn/usocket/tags/0.5.5/usocket.lisp $
+;;;; $Id$
+;;;; $URL$
 
 ;;;; See LICENSE for licensing information.
 
@@ -99,19 +99,17 @@ be initiated from remote sockets."))
   ((connected-p :type boolean
                 :accessor connected-p
                 :initarg :connected-p)
-   #+(or cmu
-         scl
-         lispworks
+   #+(or cmu scl lispworks mcl
          (and clisp ffi (not rawsock)))
    (%open-p     :type boolean
                 :accessor %open-p
                 :initform t
-		:documentation "Flag to indicate if usocket is open,
+                :documentation "Flag to indicate if usocket is open,
 for GC on implementions operate on raw socket fd.")
-   #+(or lispworks
+   #+(or lispworks mcl
          (and clisp ffi (not rawsock)))
    (recv-buffer :documentation "Private RECV buffer.")
-   #+lispworks
+   #+(or lispworks mcl)
    (send-buffer :documentation "Private SEND buffer."))
   (:documentation "UDP (inet-datagram) socket"))
 
@@ -130,7 +128,7 @@ for GC on implementions operate on raw socket fd.")
 (defun make-socket (&key socket)
   "Create a usocket socket type from implementation specific socket."
   (unless socket
-    (error 'invalid-socket))
+    (error 'invalid-socket-error))
   (make-stream-socket :socket socket))
 
 (defun make-stream-socket (&key socket stream)
@@ -504,7 +502,7 @@ to a vector quad."
                         (dotted-quad-to-vector-quad host))))
               (if (and ip (= 4 (length ip)))
                   (host-byte-order ip)
-                (host-to-hbo (get-host-by-name host)))))
+                  (host-to-hbo (get-host-by-name host)))))
     ((or (vector t 4)
          (array (unsigned-byte 8) (4)))
      (host-byte-order host))
